@@ -17,16 +17,16 @@ SEXP wrapped_intsxp(void *len) {
   return Rf_allocVector(INTSXP, *xlen);
 }
 
-SEXP wrapped_realsxp(void *len) {
-  R_xlen_t *xlen = (R_xlen_t*) len;
-  return Rf_allocVector(REALSXP, *xlen);
-}
-
 SEXP wrapped_lglsxp(void *len) {
   R_xlen_t *xlen = (R_xlen_t*) len;
   return Rf_allocVector(LGLSXP, *xlen);
 }
 
+SEXP wrapped_realsxp(void *len) {
+  R_xlen_t *xlen = (R_xlen_t*) len;
+  return Rf_allocVector(REALSXP, *xlen);
+}
+
 SEXP wrapped_strsxp(void *len) {
   R_xlen_t *xlen = (R_xlen_t*) len;
   return Rf_allocVector(STRSXP, *xlen);
@@ -42,16 +42,6 @@ SEXP wrapped_mkchar(void *data) {
   return Rf_mkChar(*c);
 }
 
-SEXP wrapped_mkchar_utf8(void *data) {
-  const char **c = (const char **) data;
-  return Rf_mkCharCE(*c, CE_UTF8);
-}
-
-SEXP wrapped_mkchar_len_utf8(void *data) {
-  struct safe_mkchar_len_data *c = (struct safe_mkchar_len_data*) data;
-  return Rf_mkCharLenCE(c->c, c->len, CE_UTF8);
-}
-
 SEXP wrapped_mkstring(void *data) {
   const char **c = (const char **) data;
   return Rf_mkString(*c);
@@ -81,14 +71,3 @@ SEXP wrapped_mknamed_vec(void *data) {
   const char ***rdata = (const char ***) data;
   return Rf_mkNamed(VECSXP, *rdata);
 }
-
-SEXP wrapped_setattrib(void *data) {
-  struct safe_setattrib_data *rdata = (struct safe_setattrib_data *) data;
-  return Rf_setAttrib(rdata->x, rdata->sym, rdata->val);
-}
-
-SEXP wrapped_xlengthgets(void *data) {
-  struct safe_xlengthgets_data *rdata =
-    (struct safe_xlengthgets_data *) data;
-  return Rf_xlengthgets(rdata->x, rdata->len);
-}
