@@ -564,14 +564,14 @@ test_that("double to INT(64, *)", {
     as.data.frame(read_parquet(tmp))
   })
 
-  d <- data.frame(d = c(2^63-1, 2^64))
+  d <- data.frame(d = c(9223372036854770000, 9223372036854774273))
   expect_snapshot(error = TRUE, {
     write_parquet(d, tmp, schema = parquet_schema("INT_64"))
-  })
-  d <- data.frame(d = c(-2^63, -2^64 - 1))
+  }, transform = redact_maxint64)
+  d <- data.frame(d = -c(9223372036854770000, 9223372036854774273))
   expect_snapshot(error = TRUE, {
     write_parquet(d, tmp, schema = parquet_schema("INT_64"))
-  })
+  }, transform = redact_maxint64)
 
   d <- data.frame(d = as.double(c(0:5, NA)))
   write_parquet(d, tmp, schema = parquet_schema("UINT_64"))
@@ -580,7 +580,7 @@ test_that("double to INT(64, *)", {
     as.data.frame(read_parquet(tmp))
   })
 
-  d <- data.frame(d = c(2^64-1, 2^65))
+  d <- data.frame(d = c(18446744073709450591.0, 18446744073709550592.0))
   expect_snapshot(error = TRUE, {
     write_parquet(d, tmp, schema = parquet_schema("UINT_64"))
   })
